/*
* This code is licensed under the Creative Commons Attribution-NonCommercial-ShareAlike 4.0 International (CC BY-NC-SA 4.0) License.
* You are free to share, copy, and redistribute the material in any medium or format, and adapt, remix, transform, and build upon the material
* as long as you give appropriate credit, do not use the material for commercial purposes, and distribute your contributions under the same license.
* 
* Note: Some portions of this code related only to joysticks, including calculations and ideas, have been taken and modified from Teaching Tech's code.
* Any changes and modifications made to the original code parts have been done to fit the specific requirements of this project.
* 
* Created by Areg Khalatyan in 2024 as part of a project in Fab Academy.
* 
* License details: https://creativecommons.org/licenses/by-nc-sa/4.0/
*/
#include <FastLED.h>
#include <BleKeyboard.h>
#include <EEPROM.h>

BleKeyboard bleKeyboard;

#define NUM_LEDS 6
#define DATA_PIN 2
CRGB leds[NUM_LEDS];
#define TOUCH_PIN_4 T4
#define TOUCH_PIN_5 T5
#define TOUCH_PIN_6 T6
#define TOUCH_PIN_7 T7
#define TOUCH_PIN_8 T8
#define TOUCH_THRESHOLD 50
int touchPins[6] = { T9, T8, T7, T6, T5, T4 };

int paletteIndex = 0;
bool lightshow = false;

//Light Show
#define UPDATES_PER_SECOND 100
CRGBPalette16 currentPalette;
int wholePaletteIndex = 0;
TBlendType currentBlending;
extern CRGBPalette16 myRedWhiteBluePalette;
extern const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM;
CRGBPalette16 pallettes[] = { RainbowColors_p, RainbowStripeColors_p, CloudColors_p, PartyColors_p, myRedWhiteBluePalette_p };

bool waitForString = true;

//This part is for debugging written by Teaching Tech. And these are his original comments.
//Joystick
// Debugging
// 0: Debugging off. Set to this once everything is working.
// 1: Output raw joystick values. 0-1023 raw ADC 10-bit values
// 2: Output centered joystick values. Values should be approx -500 to +500, jitter around 0 at idle.
// 3: Output centered joystick values. Filtered for deadzone. Approx -500 to +500, locked to zero at idle.
// 4: Output translation and rotation values. Approx -800 to 800 depending on the parameter.
// 5: Output debug 4 and 5 side by side for direct cause and effect reference.
int debug = 0;

//This part is for directions written by Teaching Tech. And these are his original comments.
//Here he only modify directions for his device.
// Direction
// Modify the direction of translation/rotation depending on preference. This can also be done per application in the 3DConnexion software.
// Switch between true/false as desired.
bool invX = false;   // pan left/right
bool invY = false;   // pan up/down
bool invZ = true;    // zoom in/out
bool invRX = true;   // Rotate around X axis (tilt front/back)
bool invRY = false;  // Rotate around Y axis (tilt left/right)
bool invRZ = true;   // Rotate around Z axis (twist left/right)

int threshold = 500;


int PINLIST[8] = {
   //This stroke also written by Teaching Tech for The positions of the reads
  36,  // X-axis A
  39,  // Y-axis A
  34,  // X-axis B
  35,  // Y-axis B
  26,  // X-axis C
  25,  // Y-axis C
  4,  // X-axis D
  15,  // Y-axis D
};//I just changed the pins for my microcontroller.
//This stroke also written by Teaching Tech for deadzone to filter out unintended movements. Recommended to have this as small as possible for  to allow smaller knob range of motion.
int DEADZONE = 200;  //I changed this value from 20 - 200 because I don't need a very precise value. On the contrary, I don’t need false positives.


struct MacroSettings {
  char keys[19][20];  // Keys for Macross
};

bool keysPressed[12] = { false, false, false, false, false, false, false, false, false, false, false, false };

// This stroke also written by Teaching Tech for define axes are matched to pin order.
#define AX 0
#define AY 1
#define BX 2
#define BY 3
#define CX 4
#define CY 5
#define DX 6
#define DY 7

//This stroke also writen by Teaching Tech for Centerpoint variable to be populated during setup routine.
int centerPoints[8];
int centerPoints[8];

//Here starts the code that Teaching Tech wrote for Function to read and store analogue voltages for each joystick axis.
void readAllFromJoystick(int *rawReads) {
  for (int i = 0; i < 8; i++) {
    rawReads[i] = analogRead(PINLIST[i]);
  }
}

MacroSettings savedSettings;

void saveSettings() {
  EEPROM.put(0, savedSettings);  // Writing settings to EEPROM
  EEPROM.commit();               // Committing the write
  Serial.println("Настройки сохранены.");
}

void loadSettings() {
  EEPROM.get(0, savedSettings);  // Loading settings from EEPROM
}

void clearEEPROM() {
  for (int i = 0; i < sizeof(savedSettings); i++) {  // Clear only the size of the structure
    EEPROM.write(i, 0);
  }
  EEPROM.commit();  
  Serial.println("EEPROM очищена.");
  strcpy(savedSettings.keys[0], "Ctrl x");
  strcpy(savedSettings.keys[1], "Ctrl v");
  strcpy(savedSettings.keys[2], "Ctrl c");
  strcpy(savedSettings.keys[3], "Ctrl z");
  strcpy(savedSettings.keys[4], "Ctrl Shift z");
  strcpy(savedSettings.keys[5], "");
  strcpy(savedSettings.keys[6], "Ctrl right");
  strcpy(savedSettings.keys[7], "Ctrl left");
  strcpy(savedSettings.keys[8], "Ctrl down");
  strcpy(savedSettings.keys[9], "Ctrl up");
  strcpy(savedSettings.keys[10], "Shift z");
  strcpy(savedSettings.keys[11], "z");
  strcpy(savedSettings.keys[12], "up");
  strcpy(savedSettings.keys[13], "down");
  strcpy(savedSettings.keys[14], "right");
  strcpy(savedSettings.keys[15], "down");
  strcpy(savedSettings.keys[16], "Alt right");
  strcpy(savedSettings.keys[17], "Alt left");
  strcpy(savedSettings.keys[18], "lighton");
  saveSettings();

}

void setup() {
  EEPROM.begin(sizeof(savedSettings));  // Initialize EEPROM with the size of the structure
  delay(2000);                          // Delay check - allows reprogramming on accidental over power with LED
  Serial.begin(115200);
  Serial.println("Starting BLE operation!");
  bleKeyboard.begin();
  FastLED.addLeds<WS2812B, DATA_PIN, RGB>(leds, NUM_LEDS);  // Typical order GRB

  currentPalette = RainbowColors_p;
  currentBlending = LINEARBLEND;

  // Read idle/centre positions for joysticks.
  readAllFromJoystick(centerPoints);
  readAllFromJoystick(centerPoints);


  loadSettings();
  Serial.print("keys 18:");
  Serial.println(savedSettings.keys[18]);
  if (strcmp(savedSettings.keys[18], "lighton") == 0) {
    sendMacro("lightshow");
    Serial.println("turn on light show");
  }
  for (int i = 0; i < 19; i++) {
    Serial.println(savedSettings.keys[i]);
  }
  
}

void loop() {
  if (lightshow) {
    //ChangePalettePeriodically();
    currentPalette = RainbowStripeColors_p;
    currentBlending = LINEARBLEND;
    FillLEDsFromPaletteColors(paletteIndex);
    paletteIndex++;
    FastLED.show();
    FastLED.delay(1000 / UPDATES_PER_SECOND);
  }
  for (int i = 0; i < 6; i++) {
    int touchValue = touchRead(touchPins[i]);
    if (touchValue < TOUCH_THRESHOLD) {
      if (bleKeyboard.isConnected()) {
        Serial.println("Keyboard connected.");
        if (i != 5) {
          sendMacro(savedSettings.keys[i]);
        }
      }
      if (i == 5) {
        sendMacro(savedSettings.keys[i]);
      }
      for (int j = 0; j < 6; j++) {
        if (j != i) {
          leds[j] = CRGB::Black;
        }
      }
      leds[i] = CRGB::White;
      FastLED.show();
      delay(100);
      leds[i] = CRGB::Black;
      FastLED.show();
    }
  }
  //Here begins the code that Teaching Tech wrote for debugging.
  int rawReads[8], centered[8];
  // Joystick values are read. 0-1023
  readAllFromJoystick(rawReads);
  // Report back 0-1023 raw ADC 10-bit values if enabled
  if (debug == 1) {
    Serial.print("AX:");
    Serial.print(rawReads[0]);
    Serial.print(",");
    Serial.print("AY:");
    Serial.print(rawReads[1]);
    Serial.print(",");
    Serial.print("BX:");
    Serial.print(rawReads[2]);
    Serial.print(",");
    Serial.print("BY:");
    Serial.print(rawReads[3]);
    Serial.print(",");
    Serial.print("CX:");
    Serial.print(rawReads[4]);
    Serial.print(",");
    Serial.print("CY:");
    Serial.print(rawReads[5]);
    Serial.print(",");
    Serial.print("DX:");
    Serial.print(rawReads[6]);
    Serial.print(",");
    Serial.print("DY:");
    Serial.println(rawReads[7]);
  }

  // Subtract centre position from measured position to determine movement.
  for (int i = 0; i < 8; i++) centered[i] = rawReads[i] - centerPoints[i];  //
  // Report centered joystick values if enabled. Values should be approx -500 to +500, jitter around 0 at idle.
  if (debug == 2) {
    Serial.print("AX:");
    Serial.print(centered[0]);
    Serial.print(",");
    Serial.print("AY:");
    Serial.print(centered[1]);
    Serial.print(",");
    Serial.print("BX:");
    Serial.print(centered[2]);
    Serial.print(",");
    Serial.print("BY:");
    Serial.print(centered[3]);
    Serial.print(",");
    Serial.print("CX:");
    Serial.print(centered[4]);
    Serial.print(",");
    Serial.print("CY:");
    Serial.print(centered[5]);
    Serial.print(",");
    Serial.print("DX:");
    Serial.print(centered[6]);
    Serial.print(",");
    Serial.print("DY:");
    Serial.println(centered[7]);
  }
  // Filter movement values. Set to zero if movement is below deadzone threshold.
  for (int i = 0; i < 8; i++) {
    if (centered[i] < DEADZONE && centered[i] > -DEADZONE) centered[i] = 0;
  }
  // Report centered joystick values. Filtered for deadzone. Approx -500 to +500, locked to zero at idle
  if (debug == 3) {
    Serial.print("AX:");
    Serial.print(centered[0]);
    Serial.print(",");
    Serial.print("AY:");
    Serial.print(centered[1]);
    Serial.print(",");
    Serial.print("BX:");
    Serial.print(centered[2]);
    Serial.print(",");
    Serial.print("BY:");
    Serial.print(centered[3]);
    Serial.print(",");
    Serial.print("CX:");
    Serial.print(centered[4]);
    Serial.print(",");
    Serial.print("CY:");
    Serial.print(centered[5]);
    Serial.print(",");
    Serial.print("DX:");
    Serial.print(centered[6]);
    Serial.print(",");
    Serial.print("DY:");
    Serial.println(centered[7]);
  }

  
  int16_t transX, transY, transZ, rotX, rotY, rotZ;  // Declare movement 
  transX = -(-centered[CY] + centered[AY]) / 1;
  transY = (-centered[BY] + centered[DY]) / 1;
  if ((abs(centered[AX]) > DEADZONE) && (abs(centered[BX]) > DEADZONE) && (abs(centered[CX]) > DEADZONE) && (abs(centered[DX]) > DEADZONE)) {
    transZ = (-centered[AX] - centered[BX] - centered[CX] - centered[DX]) / 1;
    transX = 0;
    transY = 0;
  } else {
    transZ = 0;
  }
  rotX = (-centered[AX] + centered[CX]) / 1;
  rotY = (+centered[BX] - centered[DX]) / 1;
  if ((abs(centered[AY]) > DEADZONE) && (abs(centered[BY]) > DEADZONE) && (abs(centered[CY]) > DEADZONE) && (abs(centered[DY]) > DEADZONE)) {
    rotZ = (+centered[AY] + centered[BY] + centered[CY] + centered[DY]) / 2;
    rotX = 0;
    rotY = 0;
  } else {
    rotZ = 0;
  }

  // Alter speed to suit user preference - Use 3DConnexion slider instead for V2.
  //transX = transX/100*speed;
  //transY = transY/100*speed;
  //transZ = transZ/100*speed;
  //rotX = rotX/100*speed;
  //rotY = rotY/100*speed;
  //rotZ = rotZ/100*speed;
  // Invert directions if needed
  if (invX == true) { transX = transX * -1; };
  if (invY == true) { transY = transY * -1; };
  if (invZ == true) { transZ = transZ * -1; };
  if (invRX == true) { rotX = rotX * -1; };
  if (invRY == true) { rotY = rotY * -1; };
  if (invRZ == true) { rotZ = rotZ * -1; };

  // Report translation and rotation values if enabled. Approx -800 to 800 depending on the parameter.
  if (debug == 4) {
    Serial.print("TX:");
    Serial.print(transX);
    Serial.print(",");
    Serial.print("TY:");
    Serial.print(transY);
    Serial.print(",");
    Serial.print("TZ:");
    Serial.print(transZ);
    Serial.print(",");
    Serial.print("RX:");
    Serial.print(rotX);
    Serial.print(",");
    Serial.print("RY:");
    Serial.print(rotY);
    Serial.print(",");
    Serial.print("RZ:");
    Serial.println(rotZ);
  }//This ends the code that Teaching Tech wrote for debugging. didn't change anything here.

        //This part of code i wrote for using joystick values to press and release the keys.
        //Press the corresponding keys
  int axes[6] = { 0, 0, 0, 0, 0, 0 };

  if (transX > threshold) {
    axes[0] = 1;
  } else if (transX < -threshold) {
    axes[0] = -1;
  }
  if (transY > threshold) {
    axes[1] = 1;
  } else if (transY < -threshold) {
    axes[1] = -1;
  }
  if (transZ > threshold) {
    axes[2] = 1;
  } else if (transZ < -threshold) {
    axes[2] = -1;
  }
  if (rotX > threshold) {
    axes[3] = 1;
  } else if (rotX < -threshold) {
    axes[3] = -1;
  }
  if (rotY > threshold) {
    axes[4] = 1;
  } else if (rotY < -threshold) {
    axes[4] = -1;
  }
  if (rotZ > threshold) {
    axes[5] = 1;
  } else if (rotZ < -threshold) {
    axes[5] = -1;
  }

  for (int i = 0; i < 6; i++) {
    if (axes[i] == 0) {
      if (keysPressed[i * 2]) {
        releaseKeys(savedSettings.keys[i * 2]);
        keysPressed[i * 2] = false;
      }
      if (keysPressed[i * 2 + 1]) {
        releaseKeys(savedSettings.keys[i * 2 + 1]);
        keysPressed[i * 2 + 1] = false;
      }
    }
  }
  for (int i = 0; i < 6; i++) {
    if (axes[i] == 1 && !keysPressed[i * 2]) {
      pressKeys(savedSettings.keys[6 + i * 2]);
      keysPressed[i * 2] = true;
    }
    if (axes[i] == -1 && !keysPressed[i * 2 + 1]) {
      pressKeys(savedSettings.keys[6 + i * 2 + 1]);
      keysPressed[i * 2 + 1] = true;
    }
  }

  //Here begins the code that Teaching Tech wrote for debugging.
        // Report debug 4 and 5 info side by side for direct reference if enabled. Very useful if you need to alter which inputs are used in the arithmatic above.
  if (debug == 5) {
    Serial.print("AX:");
    Serial.print(centered[0]);
    Serial.print(",");
    Serial.print("AY:");
    Serial.print(centered[1]);
    Serial.print(",");
    Serial.print("BX:");
    Serial.print(centered[2]);
    Serial.print(",");
    Serial.print("BY:");
    Serial.print(centered[3]);
    Serial.print(",");
    Serial.print("CX:");
    Serial.print(centered[4]);
    Serial.print(",");
    Serial.print("CY:");
    Serial.print(centered[5]);
    Serial.print(",");
    Serial.print("DX:");
    Serial.print(centered[6]);
    Serial.print(",");
    Serial.print("DY:");
    Serial.print(centered[7]);
    Serial.print("||");
    Serial.print("TX:");
    Serial.print(transX);
    Serial.print(",");
    Serial.print("TY:");
    Serial.print(transY);
    Serial.print(",");
    Serial.print("TZ:");
    Serial.print(transZ);
    Serial.print(",");
    Serial.print("RX:");
    Serial.print(rotX);
    Serial.print(",");
    Serial.print("RY:");
    Serial.print(rotY);
    Serial.print(",");
    Serial.print("RZ:");
    Serial.println(rotZ);
  }

  delay(50);
}// ends the code that Teaching Tech wrote for debugging. Here I just changed the delay value from 400 to 50.

void sendMacro(String keys) {
  Serial.println(keys);
  String key;
  int prevIndex = 0;
  for (int i = 0; i <= keys.length(); i++) {
    if (i == keys.length() || keys.charAt(i) == ' ') {
      key = keys.substring(prevIndex, i);
      prevIndex = i + 1;
      //lightshow = false;
      if (key == "Ctrl") {
        bleKeyboard.press(KEY_LEFT_CTRL);
      } else if (key == "Shift") {
        bleKeyboard.press(KEY_LEFT_SHIFT);
      } else if (key == "Alt") {
        bleKeyboard.press(KEY_LEFT_ALT);
      } else if (key == "Del") {
        bleKeyboard.press(KEY_DELETE);
      } else if (key == "Esc") {
        bleKeyboard.press(KEY_ESC);
      } else if (key == "lightshow") {
        if (strcmp(savedSettings.keys[18], "lighton") == 0) {
          Serial.println("lightshow");
          paletteIndex += 300;
          lightshow = true;
        } else {
          Serial.println("lightoff");
          for (int j = 0; j < 6; j++) {
            leds[j] = CRGB::Black;
          }
          FastLED.show();
          lightshow = false;
        }
      } else {
        bleKeyboard.press(key[0]);  //  single character as a key
      }
    }
  }
  delay(30);                
  bleKeyboard.releaseAll();  // Release all keys
}

void pressKeys(String keys) {  // this secrion si only for joysticks 
  Serial.println(keys);
  String key;
  int prevIndex = 0;
  for (int i = 0; i <= keys.length(); i++) {
    if (i == keys.length() || keys.charAt(i) == ' ') {
      key = keys.substring(prevIndex, i);
      prevIndex = i + 1;

      if (key == "Ctrl") {
        bleKeyboard.press(KEY_LEFT_CTRL);
      } else if (key == "Shift") {
        bleKeyboard.press(KEY_LEFT_SHIFT);
      } else if (key == "Alt") {
        bleKeyboard.press(KEY_LEFT_ALT);
      } else if (key == "Del") {
        bleKeyboard.press(KEY_DELETE);
      } else if (key == "Esc") {
        bleKeyboard.press(KEY_ESC);
      } else if (key == "left") {
        bleKeyboard.press(KEY_LEFT_ARROW);
      } else if (key == "right") {
        bleKeyboard.press(KEY_RIGHT_ARROW);
      } else if (key == "up") {
        bleKeyboard.press(KEY_UP_ARROW);
      } else if (key == "down") {
        bleKeyboard.press(KEY_DOWN_ARROW);
      } else {
        bleKeyboard.press(key[0]);  // Treat one character as a key
      }
    }
  }
}

void releaseKeys(String keys) { // this secrion si only for joysticks 
  String key;
  int prevIndex = 0;
  for (int i = 0; i <= keys.length(); i++) {
    if (i == keys.length() || keys.charAt(i) == ' ') {
      key = keys.substring(prevIndex, i);
      prevIndex = i + 1;

      if (key == "Ctrl") {
        bleKeyboard.release(KEY_LEFT_CTRL);
      } else if (key == "Shift") {
        bleKeyboard.release(KEY_LEFT_SHIFT);
      } else if (key == "Alt") {
        bleKeyboard.release(KEY_LEFT_ALT);
      } else if (key == "Del") {
        bleKeyboard.release(KEY_DELETE);
      } else if (key == "Esc") {
        bleKeyboard.release(KEY_ESC);
      } else if (key == "left") {
        bleKeyboard.release(KEY_LEFT_ARROW);
      } else if (key == "right") {
        bleKeyboard.release(KEY_RIGHT_ARROW);
      } else if (key == "up") {
        bleKeyboard.release(KEY_UP_ARROW);
      } else if (key == "down") {
        bleKeyboard.release(KEY_DOWN_ARROW);
      } else {
        bleKeyboard.release(key[0]);  // Release one keys
    }
  }
}

void serialEvent() {
  while (Serial.available()) {
    if (waitForString) {
      String input = Serial.readStringUntil('\n');
      input.trim();// Remove any leading/trailing spaces

      if (input == "reset") {
        clearEEPROM();
      } else if (input == "lightshow") {
        if (strcmp(savedSettings.keys[18], "lightoff") == 0) {
          Serial.println("lightshow");
          paletteIndex += 300;
          lightshow = true;
          strcpy(savedSettings.keys[18], "lighton");
        } else {
          Serial.println("lightoff");
          for (int j = 0; j < 6; j++) {
            leds[j] = CRGB::Black;
          }
          FastLED.show();
          lightshow = false;
          strcpy(savedSettings.keys[18], "lightoff");
        }
        saveSettings();
      } else {
        waitForString = false;  // Switch the flag, now we expect a symbol
      }
    } else {
      char input = Serial.read();
      if (input >= '0' && input <= '5') {
        int index = input - '0';
        String newKeys = "";

        while (Serial.available()) {
          char ch = Serial.read();
          if (ch == '\n') {
            break;
          }
          newKeys += ch;
        }
        newKeys = newKeys.substring(1, 20);
        char charArray[20];
        newKeys.toCharArray(charArray, 20);
        strcpy(savedSettings.keys[index], charArray);
        //savedSettings.keys[index] = charArray;
        saveSettings();
        for (int i = 0; i < 6; i++) {
          Serial.println(savedSettings.keys[i]);
        }
        Serial.print("Клавиши для сенсора ");
        Serial.print(index);
        Serial.print(" обновлены до: ");
        Serial.println(newKeys);
        waitForString = true;  // Return the flag to its original state
      }
    }
  }
}

void FillLEDsFromPaletteColors(uint8_t colorIndex) {
  uint8_t brightness = 255;

  for (int i = 0; i < NUM_LEDS; ++i) {
    leds[i] = ColorFromPalette(currentPalette, colorIndex, brightness, currentBlending);
    colorIndex += 3;
  }
}

void ChangePalettePeriodically() {
  uint8_t secondHand = (millis() / 1000) % 60;
  static uint8_t lastSecond = 99;

  if (lastSecond != secondHand) {
    lastSecond = secondHand;
    if (secondHand == 0) {
      currentPalette = RainbowColors_p;
      currentBlending = LINEARBLEND;
    }
    if (secondHand == 10) {
      currentPalette = RainbowStripeColors_p;
      currentBlending = NOBLEND;
    }
    if (secondHand == 15) {
      currentPalette = RainbowStripeColors_p;
      currentBlending = LINEARBLEND;
    }
    if (secondHand == 20) {
      SetupPurpleAndGreenPalette();
      currentBlending = LINEARBLEND;
    }
    if (secondHand == 25) {
      SetupTotallyRandomPalette();
      currentBlending = LINEARBLEND;
    }
    if (secondHand == 30) {
      SetupBlackAndWhiteStripedPalette();
      currentBlending = NOBLEND;
    }
    if (secondHand == 35) {
      SetupBlackAndWhiteStripedPalette();
      currentBlending = LINEARBLEND;
    }
    if (secondHand == 40) {
      currentPalette = CloudColors_p;
      currentBlending = LINEARBLEND;
    }
    if (secondHand == 45) {
      currentPalette = PartyColors_p;
      currentBlending = LINEARBLEND;
    }
    if (secondHand == 50) {
      currentPalette = myRedWhiteBluePalette_p;
      currentBlending = NOBLEND;
    }
    if (secondHand == 55) {
      currentPalette = myRedWhiteBluePalette_p;
      currentBlending = LINEARBLEND;
    }
  }
}

// This function fills the palette with totally random colors.
void SetupTotallyRandomPalette() {
  for (int i = 0; i < 16; ++i) {
    currentPalette[i] = CHSV(random8(), 255, random8());
  }
}

// This function sets up a palette of black and white stripes,
// using code.  Since the palette is effectively an array of
// sixteen CRGB colors, the various fill_* functions can be used
// to set them up.
void SetupBlackAndWhiteStripedPalette() {
  // 'black out' all 16 palette entries...
  fill_solid(currentPalette, 16, CRGB::Black);
  // and set every fourth one to white.
  currentPalette[0] = CRGB::White;
  currentPalette[4] = CRGB::White;
  currentPalette[8] = CRGB::White;
  currentPalette[12] = CRGB::White;
}

// This function sets up a palette of purple and green stripes.
void SetupPurpleAndGreenPalette() {
  CRGB purple = CHSV(HUE_PURPLE, 255, 255);
  CRGB green = CHSV(HUE_GREEN, 255, 255);
  CRGB black = CRGB::Black;

  currentPalette = CRGBPalette16(
    green, green, black, black,
    purple, purple, black, black,
    green, green, black, black,
    purple, purple, black, black);
}


// This example shows how to set up a static color palette
// which is stored in PROGMEM (flash), which is almost always more
// plentiful than RAM.  A static PROGMEM palette like this
// takes up 64 bytes of flash.
const TProgmemPalette16 myRedWhiteBluePalette_p PROGMEM = {
  CRGB::Red,
  CRGB::Gray,  // 'white' is too bright compared to red and blue
  CRGB::Blue,
  CRGB::Black,

  CRGB::Red,
  CRGB::Gray,
  CRGB::Blue,
  CRGB::Black,

  CRGB::Red,
  CRGB::Red,
  CRGB::Gray,
  CRGB::Gray,
  CRGB::Blue,
  CRGB::Blue,
  CRGB::Black,
  CRGB::Black
};